package admin_query

import (
	"context"
	"database/sql"
	"encoding/hex"
	"encoding/json"
	"fmt"

	"tydata-server/app/main/api/internal/svc"
	"tydata-server/app/main/api/internal/types"
	"tydata-server/common/xerr"
	"tydata-server/pkg/lzkit/crypto"
	"tydata-server/pkg/lzkit/lzUtils"

	"github.com/jinzhu/copier"
	"github.com/pkg/errors"
	"github.com/zeromicro/go-zero/core/logx"
)

type AdminGetQueryDetailByOrderIdLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewAdminGetQueryDetailByOrderIdLogic(ctx context.Context, svcCtx *svc.ServiceContext) *AdminGetQueryDetailByOrderIdLogic {
	return &AdminGetQueryDetailByOrderIdLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *AdminGetQueryDetailByOrderIdLogic) AdminGetQueryDetailByOrderId(req *types.AdminGetQueryDetailByOrderIdReq) (resp *types.AdminGetQueryDetailByOrderIdResp, err error) {

	// 获取报告信息
	queryModel, err := l.svcCtx.QueryModel.FindOneByOrderId(l.ctx, req.OrderId)
	if err != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.DB_ERROR), "报告查询, 查找报告错误: %+v", err)
	}

	var query types.AdminGetQueryDetailByOrderIdResp
	query.CreateTime = queryModel.CreateTime.Format("2006-01-02 15:04:05")
	query.UpdateTime = queryModel.UpdateTime.Format("2006-01-02 15:04:05")

	// 解密查询数据
	secretKey := l.svcCtx.Config.Encrypt.SecretKey
	key, decodeErr := hex.DecodeString(secretKey)
	if decodeErr != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.SERVER_COMMON_ERROR), "报告查询, 获取AES解密解药失败, %+v", err)
	}
	processParamsErr := ProcessQueryParams(queryModel.QueryParams, &query.QueryParams, key)
	if processParamsErr != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.SERVER_COMMON_ERROR), "报告查询, 报告参数处理失败: %v", processParamsErr)
	}
	processErr := ProcessQueryData(queryModel.QueryData, &query.QueryData, key)
	if processErr != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.SERVER_COMMON_ERROR), "报告查询, 报告结果处理失败: %v", processErr)
	}
	updateFeatureAndProductFeatureErr := l.UpdateFeatureAndProductFeature(queryModel.ProductId, &query.QueryData)
	if updateFeatureAndProductFeatureErr != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.SERVER_COMMON_ERROR), "报告查询, 报告结果处理失败: %v", updateFeatureAndProductFeatureErr)
	}
	// 复制报告数据
	err = copier.Copy(&query, queryModel)
	if err != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.SERVER_COMMON_ERROR), "报告查询, 报告结构体复制失败, %v", err)
	}
	product, err := l.svcCtx.ProductModel.FindOne(l.ctx, queryModel.ProductId)
	if err != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.SERVER_COMMON_ERROR), "报告查询, 获取商品信息失败, %v", err)
	}
	query.ProductName = product.ProductName
	return &types.AdminGetQueryDetailByOrderIdResp{
		Id:          query.Id,
		OrderId:     query.OrderId,
		UserId:      query.UserId,
		ProductName: query.ProductName,
		QueryParams: query.QueryParams,
		QueryData:   query.QueryData,
		CreateTime:  query.CreateTime,
		UpdateTime:  query.UpdateTime,
		QueryState:  query.QueryState,
	}, nil
}

// ProcessQueryData 解密和反序列化 QueryData
func ProcessQueryData(queryData sql.NullString, target *[]types.AdminQueryItem, key []byte) error {
	queryDataStr := lzUtils.NullStringToString(queryData)
	if queryDataStr == "" {
		return nil
	}

	// 解密数据
	decryptedData, decryptErr := crypto.AesDecrypt(queryDataStr, key)
	if decryptErr != nil {
		return decryptErr
	}

	// 解析 JSON 数组
	var decryptedArray []map[string]interface{}
	unmarshalErr := json.Unmarshal(decryptedData, &decryptedArray)
	if unmarshalErr != nil {
		return unmarshalErr
	}

	// 确保 target 具有正确的长度
	if len(*target) == 0 {
		*target = make([]types.AdminQueryItem, len(decryptedArray))
	}

	// 填充解密后的数据到 target
	for i := 0; i < len(decryptedArray); i++ {
		// 直接填充解密数据到 Data 字段
		(*target)[i].Data = decryptedArray[i]
	}
	return nil
}

// ProcessQueryParams解密和反序列化 QueryParams
func ProcessQueryParams(QueryParams string, target *map[string]interface{}, key []byte) error {
	// 解密 QueryParams
	decryptedData, decryptErr := crypto.AesDecrypt(QueryParams, key)
	if decryptErr != nil {
		return decryptErr
	}

	// 反序列化解密后的数据
	unmarshalErr := json.Unmarshal(decryptedData, target)
	if unmarshalErr != nil {
		return unmarshalErr
	}

	return nil
}

func (l *AdminGetQueryDetailByOrderIdLogic) UpdateFeatureAndProductFeature(productID int64, target *[]types.AdminQueryItem) error {
	// 遍历 target 数组，使用倒序遍历，以便删除元素时不影响索引
	for i := len(*target) - 1; i >= 0; i-- {
		queryItem := &(*target)[i]

		// 确保 Data 为 map 类型
		data, ok := queryItem.Data.(map[string]interface{})
		if !ok {
			return fmt.Errorf("queryItem.Data 必须是 map[string]interface{} 类型")
		}

		// 从 Data 中获取 apiID
		apiID, ok := data["apiID"].(string)
		if !ok {
			return fmt.Errorf("queryItem.Data 中的 apiID 必须是字符串类型")
		}

		// 查询 Feature
		feature, err := l.svcCtx.FeatureModel.FindOneByApiId(l.ctx, apiID)
		if err != nil {
			// 如果 Feature 查不到，也要删除当前 QueryItem
			*target = append((*target)[:i], (*target)[i+1:]...)
			continue
		}

		// 查询 ProductFeatureModel
		builder := l.svcCtx.ProductFeatureModel.SelectBuilder().Where("product_id = ?", productID)
		productFeatures, err := l.svcCtx.ProductFeatureModel.FindAll(l.ctx, builder, "")
		if err != nil {
			return fmt.Errorf("查询 ProductFeatureModel 错误: %v", err)
		}

		// 遍历 productFeatures，找到与 feature.ID 关联且 enable == 1 的项
		var featureData map[string]interface{}
		sort := 0
		for _, pf := range productFeatures {
			if pf.FeatureId == feature.Id { // 确保和 Feature 关联
				sort = int(pf.Sort)
				break // 找到第一个符合条件的就退出循环
			}
		}
		featureData = map[string]interface{}{
			"featureName": feature.Name,
			"sort":        sort,
		}

		// 更新 queryItem 的 Feature 字段（不是数组）
		queryItem.Feature = featureData
	}

	return nil
}
