package pay

import (
	"context"
	"fmt"
	"net/http"
	"strings"
	"time"
	"tydata-server/pkg/lzkit/lzUtils"

	"github.com/smartwalle/alipay/v3"

	"tydata-server/app/main/api/internal/svc"
	"tydata-server/app/main/model"

	"github.com/zeromicro/go-zero/core/logx"
	"github.com/zeromicro/go-zero/core/stores/sqlx"
)

type AlipayCallbackLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewAlipayCallbackLogic(ctx context.Context, svcCtx *svc.ServiceContext) *AlipayCallbackLogic {
	return &AlipayCallbackLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *AlipayCallbackLogic) AlipayCallback(w http.ResponseWriter, r *http.Request) error {
	notification, err := l.svcCtx.AlipayService.HandleAliPaymentNotification(r)
	if err != nil {
		logx.Errorf("支付宝支付回调，%v", err)
		return nil
	}

	// 根据订单号前缀判断订单类型
	orderNo := notification.OutTradeNo
	if strings.HasPrefix(orderNo, "Q_") {
		// 查询订单处理
		return l.handleQueryOrderPayment(w, notification)
	} else if strings.HasPrefix(orderNo, "A_") {
		// 代理会员订单处理
		return l.handleAgentVipOrderPayment(w, notification)
	} else {
		// 兼容旧订单，假设没有前缀的是查询订单
		return l.handleQueryOrderPayment(w, notification)
	}
}

// 处理查询订单支付
func (l *AlipayCallbackLogic) handleQueryOrderPayment(w http.ResponseWriter, notification *alipay.Notification) error {
	order, findOrderErr := l.svcCtx.OrderModel.FindOneByOrderNo(l.ctx, notification.OutTradeNo)
	if findOrderErr != nil {
		logx.Errorf("支付宝支付回调，查找订单失败: %+v", findOrderErr)
		return nil
	}

	if order.Status != "pending" {
		alipay.ACKNotification(w)
		return nil
	}

	user, err := l.svcCtx.UserModel.FindOne(l.ctx, order.UserId)
	if err != nil {
		logx.Errorf("支付宝支付回调，查找用户失败: %+v", err)
		return nil
	}

	amount := lzUtils.ToAlipayAmount(order.Amount)
	if user.Inside != 1 {
		// 确保订单金额和状态正确，防止重复更新
		if amount != notification.TotalAmount {
			logx.Errorf("支付宝支付回调，金额不一致")
			return nil
		}
	}

	switch notification.TradeStatus {
	case alipay.TradeStatusSuccess:
		order.Status = "paid"
		order.PayTime = lzUtils.TimeToNullTime(time.Now())
	default:
		return nil
	}

	order.PlatformOrderId = lzUtils.StringToNullString(notification.TradeNo)
	if updateErr := l.svcCtx.OrderModel.UpdateWithVersion(l.ctx, nil, order); updateErr != nil {
		logx.Errorf("支付宝支付回调，修改订单信息失败: %+v", updateErr)
		return nil
	}

	if order.Status == "paid" {
		if asyncErr := l.svcCtx.AsynqService.SendQueryTask(order.Id); asyncErr != nil {
			logx.Errorf("异步任务调度失败: %v", asyncErr)
			return asyncErr
		}
	}

	alipay.ACKNotification(w)
	return nil
}

// 处理代理会员订单支付
func (l *AlipayCallbackLogic) handleAgentVipOrderPayment(w http.ResponseWriter, notification *alipay.Notification) error {
	agentOrder, findAgentOrderErr := l.svcCtx.AgentMembershipRechargeOrderModel.FindOneByOrderNo(l.ctx, notification.OutTradeNo)
	if findAgentOrderErr != nil {
		logx.Errorf("支付宝支付回调，查找代理会员订单失败: %+v", findAgentOrderErr)
		return nil
	}

	if agentOrder.Status != "pending" {
		alipay.ACKNotification(w)
		return nil
	}

	user, err := l.svcCtx.UserModel.FindOne(l.ctx, agentOrder.UserId)
	if err != nil {
		logx.Errorf("支付宝支付回调，查找用户失败: %+v", err)
		return nil
	}

	amount := lzUtils.ToAlipayAmount(agentOrder.Amount)
	if user.Inside != 1 {
		// 确保订单金额和状态正确，防止重复更新
		if amount != notification.TotalAmount {
			logx.Errorf("支付宝支付回调，金额不一致")
			return nil
		}
	}

	switch notification.TradeStatus {
	case alipay.TradeStatusSuccess:
		agentOrder.Status = "paid"
	default:
		return nil
	}

	if agentOrder.Status == "paid" {
		err = l.svcCtx.AgentModel.Trans(l.ctx, func(transCtx context.Context, session sqlx.Session) error {
			agentModel, err := l.svcCtx.AgentModel.FindOne(transCtx, agentOrder.AgentId)
			if err != nil {
				return fmt.Errorf("查找代理信息失败: %+v", err)
			}
			agentOrder.PlatformOrderId = lzUtils.StringToNullString(notification.TradeNo)
			if updateErr := l.svcCtx.AgentMembershipRechargeOrderModel.UpdateWithVersion(l.ctx, nil, agentOrder); updateErr != nil {
				return fmt.Errorf("修改代理会员订单信息失败: %+v", updateErr)
			}

			// 设置会员等级
			agentModel.LevelName = agentOrder.LevelName

			// 延长会员时间
			// 检查是否是同级续费并记录到日志
			isRenewal := agentModel.LevelName == agentOrder.LevelName && agentModel.MembershipExpiryTime.Valid
			if isRenewal {
				logx.Infof("代理会员续费成功，会员ID：%d，等级：%s", agentModel.Id, agentModel.LevelName)
			} else {
				logx.Infof("代理会员新购或升级成功，会员ID：%d，等级：%s", agentModel.Id, agentModel.LevelName)
			}
			agentModel.MembershipExpiryTime = lzUtils.RenewMembership(agentModel.MembershipExpiryTime)

			if updateErr := l.svcCtx.AgentModel.UpdateWithVersion(l.ctx, nil, agentModel); updateErr != nil {
				return fmt.Errorf("修改代理信息失败: %+v", updateErr)
			}
			return nil
		})
		if err != nil {
			logx.Errorf("支付宝支付回调，处理代理会员订单失败: %+v", err)
			refundErr := l.handleRefund(agentOrder)
			if refundErr != nil {
				logx.Errorf("支付宝支付回调，退款失败: %+v", refundErr)
			}
			return nil
		}
	}

	alipay.ACKNotification(w)
	return nil
}

func (l *AlipayCallbackLogic) handleRefund(order *model.AgentMembershipRechargeOrder) error {
	ctx := context.Background()
	// 退款
	if order.PaymentMethod == "wechat" {
		refundErr := l.svcCtx.WechatPayService.WeChatRefund(ctx, order.OrderNo, order.Amount, order.Amount)
		if refundErr != nil {
			return refundErr
		}
	} else {
		refund, refundErr := l.svcCtx.AlipayService.AliRefund(ctx, order.OrderNo, order.Amount)
		if refundErr != nil {
			return refundErr
		}
		if refund.IsSuccess() {
			logx.Errorf("支付宝退款成功, orderID: %d", order.Id)
			// 更新订单状态为退款
			order.Status = "refunded"
			updateOrderErr := l.svcCtx.AgentMembershipRechargeOrderModel.UpdateWithVersion(ctx, nil, order)
			if updateOrderErr != nil {
				logx.Errorf("更新订单状态失败，订单ID: %d, 错误: %v", order.Id, updateOrderErr)
				return fmt.Errorf("更新订单状态失败: %v", updateOrderErr)
			}
			return nil
		} else {
			logx.Errorf("支付宝退款失败:%v", refundErr)
			return refundErr
		}
		// 直接成功
	}
	return nil
}
